import { __read, __spreadArray } from "tslib";
import { PropBindType } from '@cloudbase/cals';
import { observable, reaction, remove, untracked } from 'mobx';
import { EventProxy } from './eventProxy';
import { generateDataBinds } from './adapter/generate';
import { debounce } from 'lodash-es';
/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */
var eventProxy = new EventProxy();
function debounceFnFactory(fn, wait) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    var id = "".concat(fn.name, ":").concat(JSON.stringify(args));
    debounceFnFactory[id] = debounceFnFactory[id] || debounce(fn.bind.apply(fn, __spreadArray([this], __read(args), false)), wait);
    return debounceFnFactory[id];
}
export var debugExprMap = observable({ node: {} });
export function resetDebugExprMap() {
    // FIXME: widgets 中 for 之下 widget 的 dispose 没有被触发，所以这边通过 remove 来删除使原来的 autorun 不再触发
    remove(debugExprMap, 'node');
    debugExprMap.node = {};
    // window['__debugExprMap'] = debugExprMap
}
export function watchForDebugExpr(_a) {
    var widgetId = _a.widgetId, subForItems = _a.subForItems, globalCtx = _a.globalCtx, getDataContextFn = _a.getDataContextFn, get$wFn = _a.get$wFn, get$scopeFn = _a.get$scopeFn;
    // FIXME: 暂时没考虑低码组件开发模式
    var dataContext = untracked(getDataContextFn);
    var $w = untracked(get$wFn);
    var $scope = untracked(get$scopeFn);
    if (!debugExprMap.node[widgetId]) {
        debugExprMap.node[widgetId] = {
            expr: '',
            result: '',
            refreshSigh: +new Date(),
        };
    }
    var forDebugExprDispose = reaction(
    // 监听 refreshSigh 的变化
    function () { var _a, _b; return [(_a = debugExprMap.node[widgetId]) === null || _a === void 0 ? void 0 : _a.refreshSigh, (_b = debugExprMap.node[widgetId]) === null || _b === void 0 ? void 0 : _b.expr]; }, function (_a) {
        var _b, _c;
        var _d = __read(_a, 2), _ = _d[0], expr = _d[1];
        if (expr === null || expr === void 0 ? void 0 : expr.trim()) {
            var fn = (_b = generateDataBinds(globalCtx, [
                {
                    type: PropBindType.expression,
                    bindDataPath: expr,
                    propertyPath: 'expr',
                },
            ])) === null || _b === void 0 ? void 0 : _b.expr;
            if (fn) {
                try {
                    var result = fn(subForItems, undefined, dataContext, $w, $scope);
                    debugExprMap.node[widgetId].result = result;
                }
                catch (e) {
                    debugExprMap.node[widgetId].result = e;
                }
            }
            else {
                debugExprMap.node[widgetId].result = '';
            }
        }
        else {
            if ((_c = debugExprMap.node) === null || _c === void 0 ? void 0 : _c[widgetId])
                debugExprMap.node[widgetId].result = '';
        }
        // 这样对于数组只会拿最后一项
        debounceFnFactory(function (widgetId) { return eventProxy.emit("done_".concat(widgetId)); }, 300, widgetId)();
    });
    return forDebugExprDispose;
}
export function evalExpr(widgetId, expr, useCache) {
    if (useCache === void 0) { useCache = false; }
    return new Promise(function (resolve) {
        if (!debugExprMap.node[widgetId])
            resolve(undefined);
        if (useCache && debugExprMap.node[widgetId].expr === expr) {
            // 表达式没变，直接返回结果
            resolve(debugExprMap.node[widgetId].result);
        }
        else {
            eventProxy.once("done_".concat(widgetId), function () {
                resolve(debugExprMap.node[widgetId].result);
            });
            // 触发 autorun
            if (!useCache && debugExprMap.node[widgetId].expr === expr) {
                debugExprMap.node[widgetId].refreshSigh = +new Date();
            }
            else {
                debugExprMap.node[widgetId].expr = expr || '';
            }
        }
    });
}
// window['__testExpr'] = evalExpr
