"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNetworkType = exports.simpleCheckAdminPortal = exports.reportEvent = exports.reportTime = exports.isEqual = exports.serde = exports.isEnableWecom = exports.setEnableWecom = exports.ENABLE_WECOM = exports.setEnablePageRoot = exports.ENABLE_PAGE_ROOT_ID = exports.setEnableObserverRef = exports.ENABLE_OBSERVER_REF = exports.generateDataContext = exports.redirectToLogin = exports.defaultGenerateLoginUrl = exports.checkAuth = exports.formatErrorMsg = exports.checkAnonymous = exports.getAuthConfig = exports.dealCache = exports.findLoginPage = exports.getScopeSlotMeta = exports.isSlot = exports.isPlainObject = exports.isEmptyObj = exports.camelcase = exports.kebabCase = exports.bindFuncObj = exports.resolveDataBinds = void 0;
var tslib_1 = require("tslib");
var cals_1 = require("@cloudbase/cals");
var global_api_1 = require("../app/global-api");
var weda_client_1 = require("@cloudbase/weda-client");
var helper_1 = require("./sso/helper");
var init_1 = require("./sso/init");
var lodash_es_1 = require("lodash-es");
var loginScope = weda_client_1.auth.loginScope, getAccessToken = weda_client_1.auth.getAccessToken;
/**
 * All data bindings are generated as functions: (forItems) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
function resolveDataBinds(dataBinds, forItems, codeContext, scopeContext, throwError, dataContext, $w) {
    var resolvedProps = {};
    for (var prop in dataBinds) {
        var fn = dataBinds[prop];
        try {
            if (codeContext === null || codeContext === void 0 ? void 0 : codeContext.$WEAPPS_COMP) {
                fn = fn.bind(codeContext.$WEAPPS_COMP);
            }
            resolvedProps[prop] = fn(forItems, codeContext === null || codeContext === void 0 ? void 0 : codeContext.event, dataContext, $w, scopeContext);
        }
        catch (e) {
            console.error('Error resolving data binding', prop, dataBinds[prop], e);
            if (throwError) {
                throw e;
            }
        }
    }
    return resolvedProps;
}
exports.resolveDataBinds = resolveDataBinds;
/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
function bindFuncObj(funcObj, context) {
    if (funcObj === void 0) { funcObj = {}; }
    return Object.keys(funcObj).reduce(function (obj, fnName) {
        var _a, _b;
        obj[fnName] = (_b = (_a = funcObj[fnName]) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, context);
        return obj;
    }, {});
}
exports.bindFuncObj = bindFuncObj;
function kebabCase(str) {
    return str.replace(/[A-Z]/g, function (match) {
        return "-".concat(match.toLowerCase());
    });
}
exports.kebabCase = kebabCase;
function camelcase(str, firstUpperCase) {
    if (firstUpperCase === void 0) { firstUpperCase = false; }
    str = str.replace(/[_-]([a-z])/g, function (l) {
        return l[1].toUpperCase();
    });
    if (firstUpperCase)
        str = str.charAt(0).toUpperCase() + str.slice(1);
    return str;
}
exports.camelcase = camelcase;
var isEmptyObj = function (obj) {
    if (!isPlainObject(obj)) {
        return false;
    }
    for (var i in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, i)) {
            return false;
        }
    }
    return true;
};
exports.isEmptyObj = isEmptyObj;
function isPlainObject(src) {
    return Object.prototype.toString.call(src) === '[object Object]';
}
exports.isPlainObject = isPlainObject;
/**
 * @deprecated
 * 历史静态配置，不再新增
 */
var SCOPE_SLOT_COMPONENT_LIB = ['tea_basis', 'tea_shop', 'crm_basis'];
/**
 * @deprecated
 * 历史静态配置，不再新增
 */
var SCOPE_SLOT_MAP = SCOPE_SLOT_COMPONENT_LIB.reduce(function (map, lib) {
    map["".concat(lib, ":TableMatching")] = {
        headerSlot: true,
        recordSlot: true,
    };
    map["".concat(lib, ":TableExpanded")] = {
        headerSlot: true,
        recordSlot: true,
    };
    map["".concat(lib, ":TabsTable")] = {
        headerSlot: true,
        recordSlot: true,
    };
    map["".concat(lib, ":Table")] = {
        headerSlot: true,
        recordSlot: true,
    };
    return map;
}, {});
function isSlot(comp) {
    return !comp.module && !comp.component;
}
exports.isSlot = isSlot;
/**
 * 判断是否为作用域插槽
 * 1、根据组件dataForm是否有属性包含 x-slot-scope 属性
 * 2、保持历史穷举法不变
 */
function getScopeSlotMeta(comp, slot, context) {
    var _a, _b, _c;
    var sourceKey = comp.module && comp.component && "".concat(comp.module, ":").concat(comp.component);
    var map = SCOPE_SLOT_MAP[sourceKey];
    var res = {
        isScopeSlot: (map === null || map === void 0 ? void 0 : map[slot]) || false,
        useScopeWidgetData: false,
        config: undefined,
    };
    var info = (_a = context === null || context === void 0 ? void 0 : context.componentsInfoMap) === null || _a === void 0 ? void 0 : _a[sourceKey];
    function updateScopeSlotMeta(prop, config) {
        var _a;
        res.isScopeSlot = true;
        res.useScopeWidgetData = true;
        res.config = tslib_1.__assign({}, config);
        if (sourceKey === 'CLOUDBASE_BUSSINESS:ModelTable') {
            res.useScopeWidgetData = false;
        }
        else if (comp.module === cals_1.OFFICIAL_LIB_KEY || comp.module === 'weda') {
            var gt = false;
            var properties = ((_a = info.properties) === null || _a === void 0 ? void 0 : _a.properties) || info.properties;
            if (comp.component === 'WdTable' && properties.records) {
                gt = true;
            }
            if (!gt) {
                res.useScopeWidgetData = false;
            }
        }
        /**
         * 首先 版本号确定使用新版
         * 其次 若无 for 配置 默认补足配置
         */
        if (res.useScopeWidgetData) {
            if ((sourceKey === "".concat(cals_1.OFFICIAL_LIB_KEY, ":WdTable") || sourceKey === "weda:WdTable") &&
                (prop === 'cell' || prop === 'cell__custom__option') &&
                !res.config['x-slot-directives']) {
                res.config['x-slot-directives'] = {
                    for: 'records',
                    /**
                     * 没想好是记录 key 标识，用 find 去找
                     * 还是直接用 index 来
                     */
                    _index: 'recordIndex',
                };
            }
        }
    }
    try {
        var dataForm = (info || {}).dataForm;
        if (typeof dataForm === 'object') {
            /**
             * 优先精确匹配
             * 然后模糊匹配
             */
            if ((_b = dataForm[slot]) === null || _b === void 0 ? void 0 : _b['x-slot-scope']) {
                updateScopeSlotMeta(slot, dataForm[slot]);
            }
            else {
                // eslint-disable-next-line no-restricted-syntax
                for (var prop in dataForm) {
                    if (slot.startsWith("".concat(prop, "_")) && ((_c = dataForm[prop]) === null || _c === void 0 ? void 0 : _c['x-slot-scope'])) {
                        updateScopeSlotMeta(prop, dataForm[prop]);
                        break;
                    }
                }
            }
        }
    }
    catch (e) { }
    return res;
}
exports.getScopeSlotMeta = getScopeSlotMeta;
function findLoginPage(app) {
    if (app === void 0) { app = global_api_1.app; }
    var _a = app.__internal__.getConfig().pages, pages = _a === void 0 ? [] : _a;
    return pages.find(function (item) { return item.type === 'login'; });
}
exports.findLoginPage = findLoginPage;
/**
 * JSON.parse封装
 * @param {*} data
 * @param {*} defaultValue 失败时返回到默认值
 * @returns
 */
var jsonParse = function (data, defaultValue) {
    try {
        return JSON.parse(data);
    }
    catch (err) {
        return defaultValue;
    }
};
/**
 * 缓存用户信息，非首次进入页面优先使用缓存
 * @param param0
 * @returns
 */
var dealCache = function (_a) {
    var key = _a.key, cacheKey = _a.cacheKey, value = _a.value, _b = _a.type, type = _b === void 0 ? 'get' : _b;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var _c, _d, clientId, _e, envId, id, _f, accessToken, userKey, cache_1, error_1;
        var _g, _h;
        var _j;
        return tslib_1.__generator(this, function (_k) {
            switch (_k.label) {
                case 0:
                    if (!localStorage || !key)
                        return [2 /*return*/, undefined];
                    _k.label = 1;
                case 1:
                    _k.trys.push([1, 4, , 5]);
                    _c = global_api_1.app.__internal__.getConfig(), _d = _c.clientId, clientId = _d === void 0 ? '' : _d, _e = _c.envId, envId = _e === void 0 ? '' : _e, id = _c.id;
                    cacheKey = "".concat(envId, "-").concat(clientId, "_").concat(cacheKey || id);
                    // 缓存值为对象，保证针对各环境各用户的信息唯一，{ [cacheKey]: { [userKey]: value } }
                    return [4 /*yield*/, global_api_1.app.cloud.getCloudInstance()];
                case 2:
                    // 缓存值为对象，保证针对各环境各用户的信息唯一，{ [cacheKey]: { [userKey]: value } }
                    _k.sent();
                    return [4 /*yield*/, getAccessToken()];
                case 3:
                    _f = ((_k.sent()) || {}).accessToken, accessToken = _f === void 0 ? '' : _f;
                    userKey = "".concat(accessToken);
                    cache_1 = jsonParse(localStorage.getItem(key), {});
                    if (type === 'get') {
                        return [2 /*return*/, (_j = cache_1[cacheKey]) === null || _j === void 0 ? void 0 : _j[userKey]];
                    }
                    else if (type === 'set') {
                        localStorage.setItem(key, JSON.stringify(tslib_1.__assign(tslib_1.__assign({}, cache_1), (_g = {}, _g[cacheKey] = (_h = {}, _h[userKey] = value, _h), _g))));
                    }
                    else if (type === 'remove') {
                        Object.keys(cache_1).forEach(function (key) {
                            if (key.startsWith(cacheKey)) {
                                delete cache_1[key];
                            }
                        });
                        localStorage.setItem(key, JSON.stringify(cache_1));
                    }
                    return [3 /*break*/, 5];
                case 4:
                    error_1 = _k.sent();
                    return [2 /*return*/, undefined];
                case 5: return [2 /*return*/];
            }
        });
    });
};
exports.dealCache = dealCache;
var getAuthConfigRes = function (app) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var res, settingData_1, e_1;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 3]);
                return [4 /*yield*/, app.cloud.callWedaApi({
                        action: 'DescribeRuntimeResourceStrategy',
                        data: {
                            ResourceType: 'app',
                            ResourceId: app.id,
                        },
                    })];
            case 1:
                res = _a.sent();
                settingData_1 = {};
                // 云api不支持map只能传字符串，需要转换
                res === null || res === void 0 ? void 0 : res.forEach(function (item) {
                    settingData_1[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
                });
                (0, exports.dealCache)({ key: 'AUTH_CONFIG', cacheKey: app.id, value: settingData_1, type: 'set' });
                return [2 /*return*/, settingData_1];
            case 2:
                e_1 = _a.sent();
                return [2 /*return*/, {
                        NeedLogin: false,
                        RejectStrategy: 'show_warning',
                    }];
            case 3: return [2 /*return*/];
        }
    });
}); };
function getAuthConfig(app) {
    if (app === void 0) { app = global_api_1.app; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var localCache;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, (0, exports.dealCache)({ key: 'AUTH_CONFIG', cacheKey: app.id, type: 'get' })];
                case 1:
                    localCache = _a.sent();
                    if (localCache) {
                        // 优先使用缓存，后更新缓存
                        setTimeout(function () {
                            getAuthConfigRes(app);
                        }, 0);
                        return [2 /*return*/, localCache];
                    }
                    return [4 /*yield*/, getAuthConfigRes(app)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.getAuthConfig = getAuthConfig;
var getAccessPermissionRes = function (_a) {
    var app = _a.app, cacheKey = _a.cacheKey, appId = _a.appId;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var res, resourceInfo, value;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, app.cloud.callWedaApi({
                        action: 'DescribeResourcesPermission',
                        data: {
                            ResourceType: "app",
                            ResourceIdList: [cacheKey],
                            AppResourceId: appId,
                        },
                    })];
                case 1:
                    res = _b.sent();
                    if (Array.isArray(res)) {
                        resourceInfo = res.find(function (i) { return i.ResourceId === cacheKey; });
                        value = {
                            isAccess: !!(resourceInfo === null || resourceInfo === void 0 ? void 0 : resourceInfo.IsAccess),
                            roleId: resourceInfo === null || resourceInfo === void 0 ? void 0 : resourceInfo.RoleId,
                        };
                        (0, exports.dealCache)({ key: 'RENDER_ACCESS', cacheKey: cacheKey, value: value, type: 'set' });
                        return [2 /*return*/, value];
                    }
                    else {
                        throw new Error("\u9274\u6743\u63A5\u53E3\u8FD4\u56DE\u53C2\u6570\u4E0D\u6B63\u786E - ".concat(JSON.stringify(res)));
                    }
                    return [2 /*return*/];
            }
        });
    });
};
function getAccessPermission(app, appId, pageId) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var cacheKey, localCache;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    cacheKey = "".concat(appId, "-").concat(pageId);
                    return [4 /*yield*/, (0, exports.dealCache)({ key: 'RENDER_ACCESS', cacheKey: cacheKey, type: 'get' })];
                case 1:
                    localCache = _a.sent();
                    if (localCache !== undefined) {
                        // 延迟获取接口数据，让其他任务先执行
                        setTimeout(function () {
                            getAccessPermissionRes({ app: app, cacheKey: cacheKey, appId: appId });
                        }, 0);
                        return [2 /*return*/, localCache];
                    }
                    return [4 /*yield*/, getAccessPermissionRes({ app: app, cacheKey: cacheKey, appId: appId })];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
function checkAnonymous() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var isAnonymous, _a, scope, accessToken, e_2;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    isAnonymous = true;
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, Promise.all([loginScope(), getAccessToken()])];
                case 2:
                    _a = tslib_1.__read.apply(void 0, [_b.sent(), 2]), scope = _a[0], accessToken = _a[1].accessToken;
                    if (accessToken && scope !== 'anonymous') {
                        isAnonymous = false;
                    }
                    return [3 /*break*/, 4];
                case 3:
                    e_2 = _b.sent();
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/, isAnonymous];
            }
        });
    });
}
exports.checkAnonymous = checkAnonymous;
function formatErrorMsg(e) {
    var msg = (e === null || e === void 0 ? void 0 : e.error_description) || (e === null || e === void 0 ? void 0 : e.message);
    var uri = e === null || e === void 0 ? void 0 : e.error_uri;
    msg = "".concat(msg).concat(uri ? "-".concat(uri) : '');
    if (msg.startsWith('Failed to fetch')) {
        msg = "\u7F51\u7EDC\u6545\u969C, \u8BF7\u68C0\u67E5\u672C\u5730\u7F51\u7EDC\u8FDE\u63A5\u662F\u5426\u6B63\u5E38(".concat(msg, ")");
    }
    return msg;
}
exports.formatErrorMsg = formatErrorMsg;
/**
 * 检查页面权限
 **/
function checkAuth(app, appId, $page, generateLoginUrl) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var loginConfigVersion, requestList, _c, accessData, authConfig, isAnonymous, e_3;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    app.showNavigationBarLoading();
                    loginConfigVersion = app.__internal__.getConfig().loginConfigVersion;
                    requestList = [getAccessPermission(app, appId, $page.id)];
                    // 有登录配置版本号时，才拉取自定义登录配置进行校验
                    if (loginConfigVersion) {
                        requestList.push(getAuthConfig(app));
                        requestList.push(checkAnonymous());
                    }
                    _d.label = 1;
                case 1:
                    _d.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, Promise.all(requestList)];
                case 2:
                    _c = tslib_1.__read.apply(void 0, [_d.sent(), 3]), accessData = _c[0], authConfig = _c[1], isAnonymous = _c[2];
                    app.hideNavigationBarLoading();
                    if (loginConfigVersion && isAnonymous) {
                        if (authConfig.NeedLogin || (!(accessData === null || accessData === void 0 ? void 0 : accessData.isAccess) && authConfig.RejectStrategy === 'to_login')) {
                            redirectToLogin(generateLoginUrl);
                            return [2 /*return*/, false];
                        }
                    }
                    if (!(accessData === null || accessData === void 0 ? void 0 : accessData.isAccess)) {
                        app.showToast({
                            title: ['-2', '-4'].includes(accessData === null || accessData === void 0 ? void 0 : accessData.roleId) ? '默认访客无权限' : '页面无访问权限',
                            icon: 'error',
                        });
                    }
                    return [2 /*return*/, accessData === null || accessData === void 0 ? void 0 : accessData.isAccess];
                case 3:
                    e_3 = _d.sent();
                    app.showModal({
                        title: '页面鉴权失败',
                        content: formatErrorMsg(e_3),
                        showCancel: false,
                    });
                    // @ts-ignore
                    (_b = (_a = window._aegis) === null || _a === void 0 ? void 0 : _a.error) === null || _b === void 0 ? void 0 : _b.call(_a, 'CUSTOM_ERROR:', e_3);
                    return [2 /*return*/, false];
                case 4: return [2 /*return*/];
            }
        });
    });
}
exports.checkAuth = checkAuth;
var defaultGenerateLoginUrl = function () {
    return getLoginUrl();
};
exports.defaultGenerateLoginUrl = defaultGenerateLoginUrl;
function getLoginUrl() {
    var app = global_api_1.app;
    var _a = app.__internal__.getConfig(), envId = _a.envId, id = _a.id, clientId = _a.clientId, loginConfigVersion = _a.loginConfigVersion;
    return "".concat(location.origin, "/__auth/?redirect_uri=").concat(encodeURIComponent(location.href), "&app_id=").concat(id, "&env_id=").concat(envId, "&client_id=").concat(clientId, "&config_version=").concat(loginConfigVersion);
}
function redirectToLogin(generateLoginUrl) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var app, _a, loginConfigVersion, id, url_1, alternative;
        return tslib_1.__generator(this, function (_b) {
            app = global_api_1.app;
            _a = app.__internal__.getConfig(), loginConfigVersion = _a.loginConfigVersion, id = _a.id;
            // 去登录则清空权限缓存。
            (0, exports.dealCache)({ key: 'RENDER_ACCESS', cacheKey: id, type: 'remove' });
            // 有登录配置版本号时，自定义登录开启，则跳转到自定义登录页面
            if (loginConfigVersion) {
                url_1 = generateLoginUrl({ app: app }, getLoginUrl);
                alternative = function alternative() {
                    app.redirectTo({
                        mode: 'web',
                        params: {},
                        url: url_1,
                    });
                };
                if (exports.ENABLE_WECOM) {
                    (0, init_1.tryWecomLogin)(new URL(url_1), alternative);
                }
                else {
                    alternative();
                }
            }
            else {
                app.showToast({
                    title: '用户未登录',
                    icon: 'error',
                });
            }
            return [2 /*return*/];
        });
    });
}
exports.redirectToLogin = redirectToLogin;
function generateDataContext(widget) {
    var dataContext = {};
    var _loop_1 = function () {
        var current = widget;
        // 此处采用代理的方式，是为了可以获取到最新的 _scope.dataContext 防止 dataContext 引用被重新赋值
        Object.defineProperty(dataContext, current._scope.id, {
            get: function () {
                return current._scope.dataContext;
            },
        });
        widget = widget.parent;
    };
    while (widget === null || widget === void 0 ? void 0 : widget._scope) {
        _loop_1();
    }
    return dataContext;
}
exports.generateDataContext = generateDataContext;
exports.ENABLE_OBSERVER_REF = false;
function setEnableObserverRef(value) {
    exports.ENABLE_OBSERVER_REF = value;
}
exports.setEnableObserverRef = setEnableObserverRef;
exports.ENABLE_PAGE_ROOT_ID = false;
function setEnablePageRoot(value) {
    exports.ENABLE_PAGE_ROOT_ID = value;
}
exports.setEnablePageRoot = setEnablePageRoot;
exports.ENABLE_WECOM = false;
function setEnableWecom(value) {
    exports.ENABLE_WECOM = value;
}
exports.setEnableWecom = setEnableWecom;
/**
 * 是否开启原地企微登录
 * 1. 针对单环境白名单
 * 2. 在企微环境打开
 * 3. 自定义应用（即：非企业工作台)
 */
var WECOM_ENVID_LIST = [
    'lowcode-4gzdsu5k66946492',
    'lowcode-2gay8jgh25c7b1cf',
    'lowcode-4g9b78bx94404d85',
    'lowcode-8gmx7bat3135e2de',
];
function isEnableWecom(envId) {
    var adminPortal = simpleCheckAdminPortal();
    if ((0, helper_1.inWxWork)() && !adminPortal && WECOM_ENVID_LIST.includes(envId)) {
        return true;
    }
}
exports.isEnableWecom = isEnableWecom;
function serde(a) {
    return JSON.parse(JSON.stringify(a));
}
exports.serde = serde;
var isObj = function (a) { return typeof a === 'object' && a !== null; };
function isEqual(a, b, shouldSerde) {
    if (shouldSerde === void 0) { shouldSerde = false; }
    if (shouldSerde && isObj(a) && isObj(b)) {
        // for Proxy created by mbox
        return (0, lodash_es_1.isEqual)(serde(a), serde(b));
    }
    return (0, lodash_es_1.isEqual)(a, b);
}
exports.isEqual = isEqual;
var _REPORTED = {};
function reportTime(tag, time, only) {
    var _a, _b, _c;
    if (only === void 0) { only = false; }
    var excludes = ['app-nvzcvt10', 'app-msa2ihs9'];
    var wedahostconfig = window._WedaHostConfig;
    if (!window._aegis || !tag || simpleCheckAdminPortal() || !window._aegis_inited || excludes.includes(global_api_1.app.id)) {
        return;
    }
    if (wedahostconfig && !wedahostconfig._REPORTED) {
        wedahostconfig._REPORTED = _REPORTED;
    }
    var CACHE_MAP = (wedahostconfig === null || wedahostconfig === void 0 ? void 0 : wedahostconfig._REPORTED) || _REPORTED;
    if (only && CACHE_MAP[tag]) {
        return;
    }
    CACHE_MAP[tag] = true;
    var envVersion = (((_b = (_a = global_api_1.app.__internal__) === null || _a === void 0 ? void 0 : _a.getConfig) === null || _b === void 0 ? void 0 : _b.call(_a)) || {}).envVersion;
    try {
        var t = time === undefined ? ((_c = performance === null || performance === void 0 ? void 0 : performance.now) === null || _c === void 0 ? void 0 : _c.call(performance)) || Date.now() - window._aegis_inited : time;
        window._aegis.reportTime({
            name: tag,
            duration: t,
            ext2: envVersion,
        });
        if (t > 5000 && (performance === null || performance === void 0 ? void 0 : performance.getEntries)) {
            window._aegis.report({
                msg: performance.getEntries(),
                ext2: envVersion,
                trace: 'trace',
            });
        }
    }
    catch (e) { }
}
exports.reportTime = reportTime;
function reportEvent(tag) {
    var _a, _b, _c, _d;
    var envVersion = (((_b = (_a = global_api_1.app.__internal__) === null || _a === void 0 ? void 0 : _a.getConfig) === null || _b === void 0 ? void 0 : _b.call(_a)) || {}).envVersion;
    try {
        (_d = (_c = window._aegis) === null || _c === void 0 ? void 0 : _c.reportEvent) === null || _d === void 0 ? void 0 : _d.call(_c, {
            name: tag,
            ext2: envVersion,
        });
    }
    catch (e) { }
}
exports.reportEvent = reportEvent;
function simpleCheckAdminPortal() {
    return window._WedaHostConfig && window.__POWERED_BY_QIANKUN__;
}
exports.simpleCheckAdminPortal = simpleCheckAdminPortal;
function getNetworkType(window) {
    var e_4, _a;
    var _b, _c, _d, _e, _f, _g, _h;
    var ua = ((_b = window === null || window === void 0 ? void 0 : window.navigator) === null || _b === void 0 ? void 0 : _b.userAgent) || '';
    var networkTypeStr = ua.match(/NetType\/\w+/)
        ? (_e = (_d = (_c = ua
            .match(/NetType\/\w+/)) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.toLowerCase()) === null || _e === void 0 ? void 0 : _e.replace('nettype/', '')
        : ((_h = (_g = (_f = window === null || window === void 0 ? void 0 : window.navigator) === null || _f === void 0 ? void 0 : _f.connection) === null || _g === void 0 ? void 0 : _g.effectiveType) === null || _h === void 0 ? void 0 : _h.toLowerCase()) || '';
    var types = ['2g', '3g', '4g', '5g', 'wifi'];
    try {
        for (var types_1 = tslib_1.__values(types), types_1_1 = types_1.next(); !types_1_1.done; types_1_1 = types_1.next()) {
            var type = types_1_1.value;
            if (networkTypeStr.includes(type))
                return type;
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (types_1_1 && !types_1_1.done && (_a = types_1.return)) _a.call(types_1);
        }
        finally { if (e_4) throw e_4.error; }
    }
    return networkTypeStr ? 'other' : '';
}
exports.getNetworkType = getNetworkType;
//# sourceMappingURL=common.js.map