"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.evalExpr = exports.watchForDebugExpr = exports.resetDebugExprMap = exports.debugExprMap = void 0;
var tslib_1 = require("tslib");
var cals_1 = require("@cloudbase/cals");
var mobx_1 = require("mobx");
var eventProxy_1 = require("./eventProxy");
var generate_1 = require("./adapter/generate");
var lodash_es_1 = require("lodash-es");
/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */
var eventProxy = new eventProxy_1.EventProxy();
function debounceFnFactory(fn, wait) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    var id = "".concat(fn.name, ":").concat(JSON.stringify(args));
    debounceFnFactory[id] = debounceFnFactory[id] || (0, lodash_es_1.debounce)(fn.bind.apply(fn, tslib_1.__spreadArray([this], tslib_1.__read(args), false)), wait);
    return debounceFnFactory[id];
}
exports.debugExprMap = (0, mobx_1.observable)({ node: {} });
function resetDebugExprMap() {
    // FIXME: widgets 中 for 之下 widget 的 dispose 没有被触发，所以这边通过 remove 来删除使原来的 autorun 不再触发
    (0, mobx_1.remove)(exports.debugExprMap, 'node');
    exports.debugExprMap.node = {};
    // window['__debugExprMap'] = debugExprMap
}
exports.resetDebugExprMap = resetDebugExprMap;
function watchForDebugExpr(_a) {
    var widgetId = _a.widgetId, subForItems = _a.subForItems, globalCtx = _a.globalCtx, getDataContextFn = _a.getDataContextFn, get$wFn = _a.get$wFn, get$scopeFn = _a.get$scopeFn;
    // FIXME: 暂时没考虑低码组件开发模式
    var dataContext = (0, mobx_1.untracked)(getDataContextFn);
    var $w = (0, mobx_1.untracked)(get$wFn);
    var $scope = (0, mobx_1.untracked)(get$scopeFn);
    if (!exports.debugExprMap.node[widgetId]) {
        exports.debugExprMap.node[widgetId] = {
            expr: '',
            result: '',
            refreshSigh: +new Date(),
        };
    }
    var forDebugExprDispose = (0, mobx_1.reaction)(
    // 监听 refreshSigh 的变化
    function () { var _a, _b; return [(_a = exports.debugExprMap.node[widgetId]) === null || _a === void 0 ? void 0 : _a.refreshSigh, (_b = exports.debugExprMap.node[widgetId]) === null || _b === void 0 ? void 0 : _b.expr]; }, function (_a) {
        var _b, _c;
        var _d = tslib_1.__read(_a, 2), _ = _d[0], expr = _d[1];
        if (expr === null || expr === void 0 ? void 0 : expr.trim()) {
            var fn = (_b = (0, generate_1.generateDataBinds)(globalCtx, [
                {
                    type: cals_1.PropBindType.expression,
                    bindDataPath: expr,
                    propertyPath: 'expr',
                },
            ])) === null || _b === void 0 ? void 0 : _b.expr;
            if (fn) {
                try {
                    var result = fn(subForItems, undefined, dataContext, $w, $scope);
                    exports.debugExprMap.node[widgetId].result = result;
                }
                catch (e) {
                    exports.debugExprMap.node[widgetId].result = e;
                }
            }
            else {
                exports.debugExprMap.node[widgetId].result = '';
            }
        }
        else {
            if ((_c = exports.debugExprMap.node) === null || _c === void 0 ? void 0 : _c[widgetId])
                exports.debugExprMap.node[widgetId].result = '';
        }
        // 这样对于数组只会拿最后一项
        debounceFnFactory(function (widgetId) { return eventProxy.emit("done_".concat(widgetId)); }, 300, widgetId)();
    });
    return forDebugExprDispose;
}
exports.watchForDebugExpr = watchForDebugExpr;
function evalExpr(widgetId, expr, useCache) {
    if (useCache === void 0) { useCache = false; }
    return new Promise(function (resolve) {
        if (!exports.debugExprMap.node[widgetId])
            resolve(undefined);
        if (useCache && exports.debugExprMap.node[widgetId].expr === expr) {
            // 表达式没变，直接返回结果
            resolve(exports.debugExprMap.node[widgetId].result);
        }
        else {
            eventProxy.once("done_".concat(widgetId), function () {
                resolve(exports.debugExprMap.node[widgetId].result);
            });
            // 触发 autorun
            if (!useCache && exports.debugExprMap.node[widgetId].expr === expr) {
                exports.debugExprMap.node[widgetId].refreshSigh = +new Date();
            }
            else {
                exports.debugExprMap.node[widgetId].expr = expr || '';
            }
        }
    });
}
exports.evalExpr = evalExpr;
// window['__testExpr'] = evalExpr
//# sourceMappingURL=debugExpr.js.map