"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Query = void 0;
var tslib_1 = require("tslib");
var cals_1 = require("@cloudbase/cals");
var mobx_1 = require("mobx");
var generate_1 = require("./adapter/generate");
var runtime_1 = require("./runtime");
var lodash_es_1 = require("lodash-es");
var Query = /** @class */ (function () {
    function Query(_a) {
        var schema = _a.schema, context = _a.context, options = _a.options;
        var _this = this;
        var _b, _c, _d;
        this._disposes = [];
        this._dataBinds = {};
        this._triggered = false;
        this._paramsRef = (0, mobx_1.observable)({ current: null });
        this._currentRequestKey = null;
        this._observableValue = (0, mobx_1.observable)({ data: null, error: null });
        this._debounceTrigger = (0, lodash_es_1.debounce)(this._innerTrigger, 300);
        var _e = options.looseError, looseError = _e === void 0 ? false : _e, _f = options.useScopeWidgetData, useScopeWidgetData = _f === void 0 ? false : _f;
        var $w = context.$w, $app = context.$app, $page = context.$page;
        this._schema = schema;
        this._context = context;
        if (((_b = this._schema) === null || _b === void 0 ? void 0 : _b.trigger) === 'auto') {
            this._disposes.push((0, mobx_1.autorun)(function (r) {
                try {
                    var data = _this._resolveParams();
                    if (_this._triggered) {
                        _this._debounceTrigger(data);
                    }
                }
                catch (e) {
                    console.error(e);
                }
            }, { delay: 50 }));
        }
        var baseParam = {
            dataSourceName: this._schema.dataSourceName,
        };
        if (((_c = this._schema) === null || _c === void 0 ? void 0 : _c.type) === 'sql') {
            baseParam.sqlTemplateId = this._schema.method.sqlTemplateId;
        }
        else {
            baseParam.methodName = this._schema.method.name;
        }
        var deserializeContext = ($page === null || $page === void 0 ? void 0 : $page.id)
            ? {
                page: { id: $page.id, type: cals_1.EComponentType.PAGE },
            }
            : {};
        var processedParams = ((_d = this._schema.method) === null || _d === void 0 ? void 0 : _d[':params'])
            ? (0, cals_1.deserializeDynamicMap)(deserializeContext, { ':params': this._schema.method.params })
            : (0, cals_1.deserializeDynamicMap)(deserializeContext, { params: this._schema.method.params });
        var _g = (0, generate_1.splitDynamicMap)(processedParams), staticData = _g.staticData, _h = _g.dataBinds, dataBinds = _h === void 0 ? [] : _h;
        this._paramsRef.current = staticData;
        this._dataBinds = (0, generate_1.generateDataBinds)({ $page: $page, $app: $app }, dataBinds, tslib_1.__assign(tslib_1.__assign({ looseError: looseError, useScopeWidgetData: useScopeWidgetData }, options), { key: "".concat(this.id, " method.params") }));
        this._action = function (data) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, $w.cloud.callDataSource(tslib_1.__assign(tslib_1.__assign({}, baseParam), { params: data }))];
            });
        }); };
        this._emit = function (trigger, eventData, originalEvent) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, (0, runtime_1.generateEmit)($w)(trigger, (0, generate_1.generateListnerInstances)({
                        $app: $app,
                        $page: $page,
                        actionsMap: this._context.actionsMap,
                    }, this._schema.listeners || []), eventData, {}, originalEvent, {}, {}, $w.page.id, true)];
            });
        }); };
        return this;
    }
    Object.defineProperty(Query.prototype, "id", {
        get: function () {
            var _a;
            return ((_a = this._schema) === null || _a === void 0 ? void 0 : _a.id) || '';
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "label", {
        get: function () {
            var _a;
            return ((_a = this._schema) === null || _a === void 0 ? void 0 : _a.label) || '';
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "description", {
        get: function () {
            var _a;
            return ((_a = this._schema) === null || _a === void 0 ? void 0 : _a.description) || '';
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "data", {
        get: function () {
            return this._observableValue.data;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Query.prototype, "error", {
        get: function () {
            return this._observableValue.error;
        },
        enumerable: false,
        configurable: true
    });
    Query.prototype.trigger = function (additionalScope, options) {
        if (options === void 0) { options = {}; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this._triggered = true;
                return [2 /*return*/, this._innerTrigger(this._resolveParams(), additionalScope, options)];
            });
        });
    };
    Query.prototype.reset = function () {
        this._observableValue.data = null;
        this._observableValue.error = null;
    };
    Query.prototype._innerTrigger = function (data, additionalScope, options) {
        if (options === void 0) { options = {}; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var key, res, e_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this._currentRequestKey = Date.now();
                        key = this._currentRequestKey;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this._action(data)];
                    case 2:
                        res = _a.sent();
                        if (key === this._currentRequestKey) {
                            this._observableValue.data = res;
                            this._observableValue.error = null;
                            this._emit("success", res);
                        }
                        return [2 /*return*/, res];
                    case 3:
                        e_1 = _a.sent();
                        if (key === this._currentRequestKey) {
                            this._observableValue.data = null;
                            this._observableValue.error = e_1;
                            this._emit("fail", e_1);
                        }
                        throw e_1;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    Query.prototype.destroy = function () {
        this._disposes.forEach(function (dispose) { return dispose(); });
    };
    Query.prototype._resolveParams = function () {
        var _a;
        /**
         * 这里万一其中某个字段计算失败
         * 好像会阻塞其他字段的计算
         * 从而导致 autorun 没有添加依赖
         */
        return (_a = (0, runtime_1.mergeDynamic2StaticData)((0, mobx_1.toJS)(this._paramsRef.current), this._dataBinds, {
            $w: this._context.$w,
            // may be additional scope
        })) === null || _a === void 0 ? void 0 : _a.params;
    };
    return Query;
}());
exports.Query = Query;
//# sourceMappingURL=query.js.map