"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeDynamic2StaticData = exports.generateEmit = void 0;
var tslib_1 = require("tslib");
var common_1 = require("../common");
var lodash_es_1 = require("lodash-es");
var global_api_1 = require("../../app/global-api");
var adapter_1 = require("../adapter");
var mobx_1 = require("mobx");
var widgets_1 = require("../widgets");
var weda_client_1 = require("@cloudbase/weda-client");
var DEFAULT_MAX_TIMEOUT = 10 * 1000;
var Event = /** @class */ (function () {
    function Event(_a) {
        var _b = _a.type, type = _b === void 0 ? '' : _b, _c = _a.detail, detail = _c === void 0 ? undefined : _c, _d = _a.currentTarget, currentTarget = _d === void 0 ? undefined : _d, _e = _a.target, target = _e === void 0 ? undefined : _e, _f = _a._isCapturePhase, _isCapturePhase = _f === void 0 ? false : _f, origin = _a.origin;
        function proxyWrapper(target, key) {
            try {
                return new Proxy(target, {
                    get: function (target, p) {
                        if (p !== 'id') {
                            (0, common_1.reportEvent)("event.".concat(key, ".").concat(String(p)));
                        }
                        return target[p];
                    },
                });
            }
            catch (e) {
                return target;
            }
        }
        this.type = type;
        this.detail = detail;
        this.currentTarget = proxyWrapper(currentTarget, 'currentTarget');
        this.target = proxyWrapper(target, 'target');
        this.origin = proxyWrapper(origin, 'origin');
        this._isCapturePhase = _isCapturePhase;
        return new Proxy(this, {
            get: function (target, prop) {
                switch (prop) {
                    case 'name': {
                        console.warn('[deprecated] event.name 将在未来版本放弃支持，请使用 event.type 替代');
                        return target.type;
                    }
                    case 'origin':
                    case 'target':
                    case 'currentTarget': {
                        return target[prop];
                        break;
                    }
                }
                return target[prop];
            },
        });
    }
    return Event;
}());
function emitEvent(trigger, listeners, args, context, syncCall) {
    if (listeners === void 0) { listeners = []; }
    if (context === void 0) { context = {}; }
    if (syncCall === void 0) { syncCall = false; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        function checkPageActive(pageId, currentPageContext, listener) {
            var _a = currentPageContext || {}, id = _a.id, __internal__ = _a.__internal__;
            if (pageId && id) {
                if (pageId !== id || (__internal__ && !__internal__.active)) {
                    console.error("Action error: [".concat(listener.trigger, ":").concat(listener.key, "] \u9875\u9762\u751F\u547D\u5468\u671F\u7ED3\u675F\uFF0C\u94FE\u5F0F\u8C03\u7528\u4E2D\u65AD"));
                    return false;
                }
            }
            return true;
        }
        function processListener(listener) {
            var _a, _b, _c;
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var nextEvents, res, e_2, res, showModalNextEvent, e_3, emitEventPromise_1;
                return tslib_1.__generator(this, function (_d) {
                    switch (_d.label) {
                        case 0:
                            // 当前非捕获Event，再判断冒泡行为
                            if (!(currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent._isCapturePhase) && listener.noPropagation) {
                                ((_a = args === null || args === void 0 ? void 0 : args.originalEvent) === null || _a === void 0 ? void 0 : _a.stopPropagation) && ((_b = args === null || args === void 0 ? void 0 : args.originalEvent) === null || _b === void 0 ? void 0 : _b.stopPropagation());
                            }
                            nextEvents = [
                                new Event(tslib_1.__assign(tslib_1.__assign({}, currentEvent), { type: '', detail: '', origin: currentEvent })),
                            ];
                            if (!(((listener === null || listener === void 0 ? void 0 : listener.isCapturePhase) || false) === ((currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent._isCapturePhase) || false))) return [3 /*break*/, 9];
                            /**
                             * 调用前置校验
                             * 是否仍可调用方法
                             */
                            if (!checkPageActive(pageId, $w === null || $w === void 0 ? void 0 : $w.page, listener)) {
                                return [2 /*return*/];
                            }
                            if (!((listener === null || listener === void 0 ? void 0 : listener.sourceKey) === 'platform:utils.If')) return [3 /*break*/, 5];
                            _d.label = 1;
                        case 1:
                            _d.trys.push([1, 3, , 4]);
                            return [4 /*yield*/, invokeListener(listener, currentEvent, context)];
                        case 2:
                            res = _d.sent();
                            nextEvents[0].type = "".concat(listener.key, ".").concat(res ? 'success' : 'fail');
                            return [3 /*break*/, 4];
                        case 3:
                            e_2 = _d.sent();
                            nextEvents[0].type = "".concat(listener.key, ".fail");
                            return [3 /*break*/, 4];
                        case 4:
                            nextEvents[0].detail = currentEvent.detail;
                            return [3 /*break*/, 8];
                        case 5:
                            _d.trys.push([5, 7, , 8]);
                            return [4 /*yield*/, invokeListener(listener, currentEvent, context)];
                        case 6:
                            res = _d.sent();
                            nextEvents[0].type = "".concat(listener.key, ".success");
                            nextEvents[0].detail = res;
                            if ((listener === null || listener === void 0 ? void 0 : listener.sourceKey) === "platform:".concat(weda_client_1._ACTIONS_KEY.showModal)) {
                                showModalNextEvent = new Event(tslib_1.__assign(tslib_1.__assign({}, currentEvent), { type: '', detail: res, origin: currentEvent }));
                                if (res.cancel) {
                                    showModalNextEvent.type = "".concat(listener.key, ".cancel");
                                }
                                else if (res.confirm) {
                                    showModalNextEvent.type = "".concat(listener.key, ".confirm");
                                }
                                if (showModalNextEvent.type) {
                                    nextEvents.push(showModalNextEvent);
                                }
                            }
                            return [3 /*break*/, 8];
                        case 7:
                            e_3 = _d.sent();
                            if (e_3.message) {
                                e_3.message = "".concat(((_c = currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent.currentTarget) === null || _c === void 0 ? void 0 : _c.id) || '', "[").concat(listener.trigger, ":").concat(listener.key, "] ").concat(e_3.message);
                            }
                            nextEvents[0].type = "".concat(listener.key, ".fail");
                            nextEvents[0].detail = e_3;
                            if (syncCall) {
                                throw e_3;
                            }
                            else {
                                // 之前 invoke 内部catch 了错误，不会抛错
                                console.error("Action error: [".concat(listener.trigger, "]"), e_3);
                            }
                            return [3 /*break*/, 8];
                        case 8:
                            /**
                             * 调用后置校验
                             * 是否触发后置事件
                             */
                            if (checkPageActive(pageId, $w === null || $w === void 0 ? void 0 : $w.page, listener)) {
                                emitEventPromise_1 = null;
                                nextEvents.forEach(function (nextEvent) {
                                    emitEventPromise_1 = emitEvent(nextEvent.type, listeners, tslib_1.__assign(tslib_1.__assign({}, args), { event: nextEvent }), context, syncCall);
                                });
                                return [2 /*return*/, syncCall ? emitEventPromise_1 : undefined];
                            }
                            _d.label = 9;
                        case 9: return [2 /*return*/];
                    }
                });
            });
        }
        var _a, $w, _b, pageId, targetListeners, currentEvent, promise, listenerRes, res, listenerRes_1, listenerRes_1_1, item;
        var e_1, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _a = context || {}, $w = _a.$w, _b = _a.pageId, pageId = _b === void 0 ? '' : _b;
                    targetListeners = listeners.filter(function (l) { return l.trigger === trigger; });
                    currentEvent = args === null || args === void 0 ? void 0 : args.event;
                    if (!(!targetListeners.length && typeof trigger === 'string' && /\.fail$/.test(trigger))) return [3 /*break*/, 1];
                    if (!syncCall) {
                        global_api_1.app.showToast({
                            title: "\u4E8B\u4EF6\u54CD\u5E94\u5931\u8D25\uFF1A".concat(currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent.detail),
                            icon: 'error',
                        });
                    }
                    return [3 /*break*/, 3];
                case 1:
                    promise = Promise.all(targetListeners.map(function (listener) {
                        return processListener(listener).then(function (res) { return ({ data: res, error: null }); }, 
                        /**
                         * 按说 processListener 不会报错
                         */
                        function (error) { return ({ data: null, error: error }); });
                    }));
                    if (!syncCall) return [3 /*break*/, 3];
                    return [4 /*yield*/, promise];
                case 2:
                    listenerRes = _d.sent();
                    res = [];
                    try {
                        for (listenerRes_1 = tslib_1.__values(listenerRes), listenerRes_1_1 = listenerRes_1.next(); !listenerRes_1_1.done; listenerRes_1_1 = listenerRes_1.next()) {
                            item = listenerRes_1_1.value;
                            if (item.error) {
                                throw item.error;
                            }
                            else {
                                res.push(item.data);
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (listenerRes_1_1 && !listenerRes_1_1.done && (_c = listenerRes_1.return)) _c.call(listenerRes_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    return [2 /*return*/, res.length ? res[0] : currentEvent.detail];
                case 3: return [2 /*return*/];
            }
        });
    });
}
function generateEmit($w, currentWidget, // page 节点暂未生成
injectContext, extraListeners, extraForContext) {
    if (currentWidget === void 0) { currentWidget = undefined; }
    if (injectContext === void 0) { injectContext = undefined; }
    if (extraListeners === void 0) { extraListeners = []; }
    if (extraForContext === void 0) { extraForContext = {}; }
    return function (trigger, listeners, eventData, forItems, originalEvent, scopeContext, dataContext, pageId, syncCall) {
        if (listeners === void 0) { listeners = []; }
        if (pageId === void 0) { pageId = ''; }
        if (syncCall === void 0) { syncCall = false; }
        listeners = listeners.concat(extraListeners);
        if (injectContext === null || injectContext === void 0 ? void 0 : injectContext.emit) {
            return injectContext === null || injectContext === void 0 ? void 0 : injectContext.emit(trigger, listeners, eventData, forItems, originalEvent, scopeContext, dataContext);
        }
        else {
            var event_1 = new Event({
                detail: eventData,
                type: trigger,
                target: currentWidget,
                currentTarget: currentWidget,
            });
            var forContext_1 = tslib_1.__assign(tslib_1.__assign({}, forItems), extraForContext);
            return emitEvent(event_1.type, listeners, {
                event: event_1,
            }, {
                pageId: pageId,
                forContext: forContext_1,
                scopeContext: scopeContext,
                dataContext: dataContext,
                $w: (0, mobx_1.untracked)(function () {
                    return (0, widgets_1.generateWidgetAPIContext)($w, currentWidget, forContext_1, scopeContext);
                }),
            }, syncCall);
        }
    };
}
exports.generateEmit = generateEmit;
function invokeListener(listener, event, context) {
    var _a, _b, _c;
    var _d = context || {}, $w = _d.$w, forContext = _d.forContext, _e = _d.scopeContext, scopeContext = _e === void 0 ? {} : _e, dataContext = _d.dataContext;
    var key = listener.key, instanceFunction = listener.instanceFunction, _f = listener.data, data = _f === void 0 ? {} : _f, _g = listener.dataBinds, dataBinds = _g === void 0 ? {} : _g;
    if (!instanceFunction) {
        try {
            // @ts-ignore
            (_b = (_a = window._aegis) === null || _a === void 0 ? void 0 : _a.error) === null || _b === void 0 ? void 0 : _b.call(_a, 'SYSTEM_ERROR: listener not found', listener);
            console.error('$page current hander', (_c = adapter_1.runtime === null || adapter_1.runtime === void 0 ? void 0 : adapter_1.runtime['$page']) === null || _c === void 0 ? void 0 : _c.handler);
        }
        catch (e) { }
        throw new Error("".concat(key, " handler \u672A\u627E\u5230'"));
    }
    // ToDo resolve databinds
    var action = instanceFunction;
    var resolvedData = mergeDynamic2StaticData(data, dataBinds, {
        $w: $w,
        forContext: forContext,
        codeContext: { event: event },
        scopeContext: scopeContext,
        dataContext: dataContext,
    });
    return action({
        data: resolvedData,
        event: event,
        forItems: forContext,
    });
}
function mergeDynamic2StaticData(staticData, dataBinds, context) {
    var _a = context.forContext, forContext = _a === void 0 ? {} : _a, _b = context.codeContext, codeContext = _b === void 0 ? {} : _b, _c = context.scopeContext, scopeContext = _c === void 0 ? {} : _c, _d = context.dataContext, dataContext = _d === void 0 ? {} : _d, $w = context.$w;
    var resolvedData = tslib_1.__assign({}, staticData);
    var resolvedDataBinds = (0, common_1.resolveDataBinds)(dataBinds, forContext, codeContext, scopeContext, true, dataContext, $w);
    // eslint-disable-next-line no-restricted-syntax
    for (var key in resolvedDataBinds) {
        if (resolvedDataBinds[key] && resolvedDataBinds[key].__type === 'scopedValue') {
            try {
                (0, lodash_es_1.set)(resolvedData, key, resolvedDataBinds[key].getValue(scopeContext));
            }
            catch (e) {
                (0, lodash_es_1.set)(resolvedData, key, '');
            }
        }
        else {
            (0, lodash_es_1.set)(resolvedData, key, resolvedDataBinds[key]);
        }
    }
    return resolvedData;
}
exports.mergeDynamic2StaticData = mergeDynamic2StaticData;
//# sourceMappingURL=actionHandler.js.map